% This is the main file to replicate some results of the paper: "Improved
% Bayesian Information Criterion for Mixture Model Selection"
% Authors: Arash Mehrjou, Reshad Hosseini, Babak Nadjar Araabi
% To see the result, set the ex_num variable and run the code.

clear all
close all
clc
warning('off','all');

%% This section generates different data set for different experiments of the paper
%Change the value of "ex_num" variable to a member of set {1,2,3,4} to replicate the results of each
%different experiments of section 5 in the paper.

%ex_num=1 generates the data set of experiment 1 produce the results denoted in
%Table1. To see the results for small data set(100 data points) that is denoted in Table2
%set the %variable "npoints" to 100 in the "case 1" block of the following
%switch-case.

%ex_num=2 generates the data set of experiment 1 with 100 points and
%produce the results denoted in Table2

%ex_num=3 generates the data set of experiment 1 with different separation(delta)
%values(0.5:0.2:2.7) and store the performance of each criteria in a
%separate table(Table{i}).

%ex_num=4 generates the highly overlapped data set corresponding to the
%experiment 2 of the paper and produce the results.

%ex_num=5 generates the 10-dimensional data set corresponding to experiment
%3 and produce the results.

%exp_num=6 loads the real data set(accidity) and confirms the
%mentioned results in section 5.4.

ex_num=2;%change his to see different experiments of the paper
switch ex_num
    case 1
        pp = [0.3333 0.3333];%prior probabilities for mixture components(the last one is determined by assuming the fact that prior probabilities shoudl sum to on
        mu1 = [0 -2];%mean vectors of mixture components
        mu2 = [0 0];
        mu3 = [0 2];
        mu = [mu1' mu2' mu3' ];
        covar(:,:,1) = [2 0; 0 0.2];%covariance matrices of mixture components
        covar(:,:,2) = [2 0; 0 0.2];
        covar(:,:,3) = [2 0; 0 0.2];
        npoints=900;%the number of data points that is generated for the data set
        self_generation=1;% a variable that shows if data should be generated or should be loaded(0 for realistic data set and 1 for generated data sets)
        
    case 2
        pp = [0.3333 0.3333];%prior probabilities for mixture components(the last one is determined by assuming the fact that prior probabilities shoudl sum to on
        mu1 = [0 -2];%mean vectors of mixture components
        mu2 = [0 0];
        mu3 = [0 2];
        mu = [mu1' mu2' mu3' ];
        covar(:,:,1) = [2 0; 0 0.2];%covariance matrices of mixture components
        covar(:,:,2) = [2 0; 0 0.2];
        covar(:,:,3) = [2 0; 0 0.2];
        npoints=100;%the number of data points that is generated for the data set
        self_generation=1;% a variable that shows if data should be generated or should be loaded(0 for realistic data set and 1 for generated data sets)
        
    case 3
        performance_versus_separation
        return
    case 4
        pp = [0.25 0.25 0.25];
        mu1 = [-4 -4];
        mu2 = [-4 -4];
        mu3 = [2 2];
        mu4 = [-1 -6];
        mu = [mu1' mu2' mu3' mu4' ];
        covar(:,:,1) = [1 0.5;...
            0.5 1];
        covar(:,:,2) = [6 -2;...
            -2 6];
        covar(:,:,3) = [2 -1;...
            -1 2];
        covar(:,:,4) = [0.125 0;...
            0 0.125];
        npoints=200;
        self_generation=1;
    case 5
        dim = 10;%change this to see the impact of different dimensions in the first data set
        delta=0.85*sqrt(dim);%we set the separation value in the order of sqrt(dim).
        pp=[0.33 0.33];
        mu1=zeros(1,dim);
        mu2=delta*ones(1,dim);
        mu3=2*delta*ones(1,dim);
        mu = [mu1' mu2' mu3'];
        covar(:,:,1) = eye(dim);
        covar(:,:,2) = eye(dim);
        covar(:,:,3) = eye(dim);
        npoints=1500;
        self_generation=1;
    case 6
        data=read_acidity;%load the real acidity data set
        data=data';
        self_generation=0;
end
%% In this section, the generated dataset is fed to the 5 criteria to estimate the true model through 100 runs
nCriteria=5;%The number of criteria to be evaluated. In this paper,this values is set to 5 for 5 compared critria.
nRuns=100;%The number of runs. Chosen number of mixture components by each criteria is determined in each run and their sum is finally reported.
kmin=1;%Minimum number of allowed mixture components
kmax=6;%Maxmimum number of allowed mixture components. the criteria should select the proper model in the range [kmin, kmax]
for i=1:nRuns
    tic
    if self_generation%determine if data should be generated here or load from a real dataset
        data = genmix(npoints,mu,covar,pp);%this function generate data points from a MoG pdf determined by its parameters
    end
    disp(sprintf('run number:%d',i))
    r{i}=table_generator(data,kmin,kmax);%the number of components selected by all criteria for i-th run is put in r{i}
    toc
    %pause
end
temp_table=zeros(nRuns,nCriteria);
for i=1:numel(r)
    temp_table(i,:)=r{i}; % r cell array is converted to a table
end
Table=zeros(kmax-kmin+1,nCriteria);
for k=kmin:kmax%This loop fill the final Table such its entries determine the number of times that a specific number of components is selected by a specific criterion
    for c=1:nCriteria
        Table(k-kmin+1,c)=sum(temp_table(:,c)==k); % table of selections is converted to a table whose arrays determine how many times each number of components is selected by each criterion
    end
end
fprintf('\tAIC   AICc  BIC   MML   BICi\n')
disp(T);
fprintf('\n')
