% This is an auxilliary file for the main file that replicates some results of the paper: "Improved
% Bayesian Information Criterion for Mixture Model Selection"
% Authors: Arash Mehrjou, Reshad Hosseini, Babak Nadjar Araabi
% This file estimates the different model selection criteria for the input data
% set and outputs the selected number of components by each criterion.
function selected_ncomps=Table_Generator(data,kmin,kmax)
% data is accepted as a dxN matrix
npoints=size(data,2);
input_dim = size(data,1);

compindex=0;

for ncomps=kmin:kmax% loop over the range of possible number of components
    % Set up a mixture model
    disp(sprintf('number of components:%d',ncomps));
    compindex=compindex+1;
    comps(compindex)=ncomps;
    mix = gmm(input_dim, ncomps, 'full');
    [mixmod_est ll_est D theta]=model_estimation(data,mix);%
    
    hX = D.weighting(theta, data);
    nk = sum(hX, 2);
    
    dk=input_dim*(input_dim+1)/2+input_dim;
    %nk=npoints*(mixmod_est.priors)
    dd=ncomps*dk+mixmod_est.ncentres;%ncomps*dk+mixmod_est.ncentres-1
    k=mixmod_est.ncentres;
    ll(compindex)=ll_est;
    
    %AIC criterion
    AIC(compindex)=-2*ll_est+2*dd;
    
    %AICc
    AICc(compindex)=-2*ll_est+2*npoints/(npoints-dd-1)*dd;
    
    %AICmc
    q=2;
    AICmc(compindex)=-2*ll_est+2*sum((nk./(nk-dk-dk./q)).*dk);
    
    %BIC
    BIC(compindex)=-2*ll_est+dd*log(npoints);
    
%     %BICmc
%     BICmc(compindex)=-2*ll_est+sum(dk.*log(nk))+2*sum((nk./(nk-dk-dk./q)).*dk);
    
    
    %MML
    MML(compindex)=-ll_est+0.5*dd*log(npoints)+0.5*dk*sum(log(mix.priors))+...
        0.5*dd*(1-log(12));
    
    
    %Ours
    %normindic = indic./(realmin+kron(ones(k,1),sum(indic,1)));
    Fisher_part=0;
    for i=1:mixmod_est.ncentres
        Fisher_part=Fisher_part+log(det(ecmnfish(data',mixmod_est.covars(:,:,i))));
    end
    Fisher_part;
%     for i=1:mix.ncentres
%         aux = kron(normindic(i,:),ones(input_dim,1)).*data';
%         Fisher_part=Fisher_part+log(det(ecmnfish(aux',mix.covars(:,:,i))));
%     end
    dd2 = dd; %k*dk
    BICi(compindex)=-2*ll_est+dd2*log(npoints)+...
        dk*sum(log(mix.priors))+Fisher_part-dd2*log(2*pi)-sum(log(mix.priors));
end
%ll
selected_ncomps=[comps(AIC==min(AIC)),...
                 comps(AICc==min(AICc)),...
                 comps(AICmc==min(AICmc)),...
                 comps(BIC==min(BIC)),...
                 comps(MML==min(MML)),...
                 comps(BICi==min(BICi)),...
                 ];




